module ToSpotlight
  class SpotlightTransferController < ApplicationController
    protect_from_forgery with: :exception
    before_action :authenticate_user!, except: [:api, :receive]
    skip_before_action :verify_authenticity_token, only: :receive
    helper_method :default_page_title, :admin_host?, :available_translations, :available_works
    include ActionView::Helpers::UrlHelper
    layout 'hyrax/dashboard' if Hyrax
    before_action :authenticate, except: [:index, :api, :receive]
    require 'json'
    require 'net/http'

    def api
      case params[:request_type]
      when 'fields'
        render plain: curated_fields
      when 'works'
        render partial: 'to_spotlight/spotlight_transfer/select', locals: { type_name: 'work', data: curated_works }
      when 'collections'
        render partial: 'to_spotlight/spotlight_transfer/select', locals: { type_name: 'collection', data: collections }
      else
        render plain: 'Invalid Request Type'
      end

    end

    def receive
      return unless params[:token] == token
      @transfer          = ToSpotlight::SpotlightTransfer.new get_transfer_attributes
      @transfer.mappings = params[:mappings]
      if @transfer.save
        render plain: 'Transfer Request Received'
      else
        render plain: 'Transfer Request Failed'
      end
    end

    def index
      @transfers = ToSpotlight::SpotlightTransfer.all
    end

    def approve
      now      = params[:commit].downcase.include?('now')
      transfer = ToSpotlight::SpotlightTransfer.find params[:transfer_id]
      works    = ::ActiveFedora::Base.where member_of_collection_ids_ssim: transfer.collection_id
      mappings = mapping_invert transfer.mappings
      if now
        SpotlightTransferJob.perform_later(works.to_a, mappings, transfer)
      else
        SpotlightTransferJob.set(wait_until: Date.today.end_of_day)
                            .perform_later(works.to_a, mappings, transfer)
      end
      redirect_to spotlight_transfer_index_path
    end

    private

    def token
      ToSpotlight::Engine.config['hyrax_instances']
    end

    def mapping_invert(hash)
      result = []
      test   = hash.map { |h| { h['hyrax'] => h['spotlight'].values } }
      test.reduce({}, :merge).map { |k, v| v.map { |val| [val, k] } }.each { |a| a.each { |a2| result << a2 } }
      Hash[result]
    end

    def curated_works
      Site.first.available_works
    end

    def collections
      ::Collection.all.map { |c| [c.title.first, c.id] }
    end

    def curated_fields
      {
        common_fields: (work_fields - excluded_fields - field_differences),
        differences: field_differences
      }
    end

    def work_fields
      @work_fields ||= curated_works.map { |work| work.constantize.new.attributes.keys }.flatten.uniq
    end

    def excluded_fields
      %w[id title head tail state proxy_depositor on_behalf_of arkivo_checksum label
       relative_path import_url part_of resource_type access_control_id
       representative_id thumbnail_id rendering_ids admin_set_id embargo_id
       lease_id]
    end

    def field_differences
      if @dif.nil?
        @dif = []
        curated_works.each do |work|
          @dif += (work.constantize.new.attributes.keys - common_fields)
        end
      end
      @dif.uniq
    end

    def common_fields
      @all_works_hash ||= intersect(curated_works.map { |w| w.constantize.new.attributes.keys })
    end

    def intersect a_of_a
      eval a_of_a.map { |a| a.inspect }.join(' & ')
    end

    def get_transfer_attributes
      params.require(:transfer).permit(:user,
                                       :exhibit_id,
                                       :collection_id,
                                       :spotlight_url)
    end

      #helpers
    def available_works
      @available_works ||= Hyrax::QuickClassificationQuery.new(current_user).authorized_models
    end

    def default_page_title
      'Spotlight Transfer'
    end

    def admin_host?
      false unless Settings.multitenancy.enabled rescue nil
    end

    def available_translations
      {
        'en' => 'English',
        'fr' => 'French'
      }
    end
  end
end
